/*
 * Copyright (c) 2011 Joseph Gaeddert
 * Copyright (c) 2011 Virginia Polytechnic Institute & State University
 *
 * This file is part of liquid.
 *
 * liquid is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * liquid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with liquid.  If not, see <http://www.gnu.org/licenses/>.
 */

//
// modem_demod_soft_const.c
//
// Constant soft demodulation arrays
//

#include <stdlib.h>
#include "liquid.internal.h"

// 
// phase-shift keying (PSK)
//

// psk8 soft demodulation nearest neighbors (p=2)
const unsigned char psk8_demod_soft_neighbors[16] = {
       4,   1,   // 000
       3,   0,   // 001
       3,   6,   // 010
       2,   1,   // 011
       0,   5,   // 100
       7,   4,   // 101
       2,   7,   // 110
       5,   6};  // 111

// psk16 soft demodulation nearest neighbors (p=2)
const unsigned char psk16_demod_soft_neighbors[32] = {
       1,   8,   // 0000
       0,   3,   // 0001
       3,   6,   // 0010
       2,   1,   // 0011
      12,   5,   // 0100
       7,   4,   // 0101
       2,   7,   // 0110
       5,   6,   // 0111
       9,   0,   // 1000
       8,  11,   // 1001
      14,  11,   // 1010
       9,  10,   // 1011
       4,  13,   // 1100
      12,  15,   // 1101
      10,  15,   // 1110
      13,  14};  // 1111

// psk32 soft demodulation nearest neighbors (p=2)
const unsigned char psk32_demod_soft_neighbors[64] = {
      16,   1,   // 00000
       0,   3,   // 00001
       3,   6,   // 00010
       1,   2,   // 00011
      12,   5,   // 00100
       7,   4,   // 00101
       7,   2,   // 00110
       5,   6,   // 00111
       9,  24,   // 01000
       8,  11,   // 01001
      14,  11,   // 01010
      10,   9,   // 01011
       4,  13,   // 01100
      12,  15,   // 01101
      10,  15,   // 01110
      14,  13,   // 01111
       0,  17,   // 10000
      16,  19,   // 10001
      19,  22,   // 10010
      18,  17,   // 10011
      21,  28,   // 10100
      20,  23,   // 10101
      23,  18,   // 10110
      22,  21,   // 10111
       8,  25,   // 11000
      27,  24,   // 11001
      27,  30,   // 11010
      25,  26,   // 11011
      29,  20,   // 11100
      28,  31,   // 11101
      26,  31,   // 11110
      29,  30};  // 11111

// 
// differential phase-shift keying (DPSK)
//



// 
// amplitude-shift keying (ASK)
//

// ask4 soft demodulation nearest neighbors (p=2)
const unsigned char ask4_demod_soft_neighbors[8] = {
       1,   3,   // 00
       0,   3,   // 01
       3,   1,   // 10
       2,   1};  // 11

// ask8 soft demodulation nearest neighbors (p=2)
const unsigned char ask8_demod_soft_neighbors[16] = {
       1,   3,   // 000
       0,   3,   // 001
       3,   6,   // 010
       2,   1,   // 011
       5,   7,   // 100
       4,   7,   // 101
       7,   2,   // 110
       6,   5};  // 111

// ask16 soft demodulation nearest neighbors (p=2)
const unsigned char ask16_demod_soft_neighbors[32] = {
       1,   3,   // 0000
       0,   3,   // 0001
       3,   6,   // 0010
       2,   1,   // 0011
      12,   5,   // 0100
       7,   4,   // 0101
       7,   2,   // 0110
       5,   6,   // 0111
       9,  11,   // 1000
       8,  11,   // 1001
      11,  14,   // 1010
      10,   9,   // 1011
       4,  13,   // 1100
      15,  12,   // 1101
      15,  10,   // 1110
      13,  14};  // 1111

// 
// quadrature amplitude-shift keying (QAM)
//

// qam8 soft demodulation nearest neighbors (p=3)
const unsigned char qam8_demod_soft_neighbors[24] = {
       1,   2,   3,   // 000
       0,   3,   2,   // 001
       0,   3,   6,   // 010
       1,   2,   7,   // 011
       5,   6,   7,   // 100
       4,   7,   6,   // 101
       2,   4,   7,   // 110
       3,   5,   6};  // 111

// qam16 soft demodulation nearest neighbors (p=4)
const unsigned char qam16_demod_soft_neighbors[64] = {
       1,   4,   5,   3,
       0,   5,   3,   4,
       3,   6,   7,   1,
       2,   7,   1,   5,
       0,   5,  12,   1,
       1,   4,   7,  13,
       2,   7,  14,   3,
       3,   6,   5,  15,
       9,  12,  13,   4,
       8,  13,  11,  12,
      11,  14,  15,   6,
      10,  15,   9,  13,
       8,  13,   4,   5,
       9,  12,   5,  15,
      10,  15,   6,   7,
      11,  14,   7,  13};

// qam32 soft demodulation nearest neighbors (p=4)
const unsigned char qam32_demod_soft_neighbors[128] = {
       1,   4,   5,   3,   // 00000
       0,   3,   5,   4,   // 00001
       3,   6,   7,   1,   // 00010
       2,   1,   7,   6,   // 00011
       5,   0,  12,   1,   // 00100
       4,   7,   1,  13,   // 00101
       7,   2,  14,   3,   // 00110
       6,   5,   3,  15,   // 00111
       9,  12,  24,  13,   // 01000
       8,  13,  11,  25,   // 01001
      11,  14,  26,  15,   // 01010
      10,  15,   9,  27,   // 01011
       8,  13,   4,   9,   // 01100
       9,  12,  15,   5,   // 01101
      10,  15,   6,  11,   // 01110
      11,  14,  13,   7,   // 01111
      17,  20,  21,  19,   // 10000
      16,  19,  21,  20,   // 10001
      19,  22,  23,  17,   // 10010
      18,  17,  23,  22,   // 10011
      21,  16,  28,  17,   // 10100
      20,  23,  17,  29,   // 10101
      23,  18,  30,  19,   // 10110
      22,  21,  19,  31,   // 10111
      25,  28,   8,  29,   // 11000
      24,  29,   9,  27,   // 11001
      27,  30,  10,  31,   // 11010
      26,  31,  11,  25,   // 11011
      24,  29,  20,  25,   // 11100
      25,  28,  31,  21,   // 11101
      26,  31,  22,  27,   // 11110
      27,  30,  29,  23};  // 11111

// qam64 soft demodulation nearest neighbors (p=4)
const unsigned char qam64_demod_soft_neighbors[256] = {
       1,   8,   9,   3,   // 000000
       0,   3,   9,   8,   // 000001
       3,   6,  10,  11,   // 000010
       2,   1,  11,  10,   // 000011
       5,  12,  13,   7,   // 000100
       4,   7,  13,  12,   // 000101
       7,   2,  14,  15,   // 000110
       6,   5,  15,  14,   // 000111
       0,   9,  24,   1,   // 001000
       1,   8,  11,  25,   // 001001
      11,  14,   2,  26,   // 001010
      10,   3,   9,  27,   // 001011
       4,  13,  28,   5,   // 001100
       5,  12,  15,  29,   // 001101
      15,  10,   6,  30,   // 001110
      14,   7,  13,  31,   // 001111
      24,  48,  17,  25,   // 010000
      25,  49,  16,  19,   // 010001
      19,  26,  22,  50,   // 010010
      18,  27,  51,  17,   // 010011
      28,  52,  21,  29,   // 010100
      29,  53,  20,  23,   // 010101
      23,  30,  18,  54,   // 010110
      22,  31,  55,  21,   // 010111
      16,   8,  25,  17,   // 011000
      17,   9,  24,  27,   // 011001
      18,  27,  30,  10,   // 011010
      19,  26,  11,  25,   // 011011
      20,  12,  29,  21,   // 011100
      21,  13,  28,  31,   // 011101
      22,  31,  26,  14,   // 011110
      23,  30,  15,  29,   // 011111
      33,  40,  41,  35,   // 100000
      32,  35,  41,  40,   // 100001
      35,  38,  42,  43,   // 100010
      34,  33,  43,  42,   // 100011
      37,  44,  45,  39,   // 100100
      36,  39,  45,  44,   // 100101
      39,  34,  46,  47,   // 100110
      38,  37,  47,  46,   // 100111
      32,  41,  56,  33,   // 101000
      33,  40,  43,  57,   // 101001
      43,  46,  34,  58,   // 101010
      42,  35,  41,  59,   // 101011
      36,  45,  60,  37,   // 101100
      37,  44,  47,  61,   // 101101
      47,  42,  38,  62,   // 101110
      46,  39,  45,  63,   // 101111
      56,  16,  49,  57,   // 110000
      57,  17,  48,  51,   // 110001
      51,  58,  18,  54,   // 110010
      50,  59,  19,  49,   // 110011
      60,  20,  53,  61,   // 110100
      61,  21,  52,  55,   // 110101
      55,  62,  22,  50,   // 110110
      54,  63,  23,  53,   // 110111
      48,  40,  57,  49,   // 111000
      49,  41,  56,  59,   // 111001
      50,  59,  62,  42,   // 111010
      51,  58,  43,  57,   // 111011
      52,  44,  61,  53,   // 111100
      53,  45,  60,  63,   // 111101
      54,  63,  58,  46,   // 111110
      55,  62,  47,  61};  // 111111

// qam128 soft demodulation nearest neighbors (p=4)
const unsigned char qam128_demod_soft_neighbors[512] = {
       8,   1,   9,  24,   // 0000000
       3,   9,   0,  11,   // 0000001
      10,   6,   3,  11,   // 0000010
       1,  11,   2,   9,   // 0000011
      12,   5,  13,  28,   // 0000100
       7,  13,   4,  15,   // 0000101
      14,   2,   7,  10,   // 0000110
       5,  15,   6,  13,   // 0000111
       0,  24,   9,   1,   // 0001000
       1,  11,  25,   8,   // 0001001
       2,  26,  14,  11,   // 0001010
       3,   9,  27,  10,   // 0001011
       4,  28,  13,   5,   // 0001100
       5,  15,  29,  12,   // 0001101
       6,  30,  10,  15,   // 0001110
       7,  13,  31,  14,   // 0001111
      48,  17,  24,  49,   // 0010000
      19,  49,  16,  25,   // 0010001
      50,  22,  19,  26,   // 0010010
      17,  51,  18,  27,   // 0010011
      52,  21,  28,  53,   // 0010100
      23,  53,  20,  29,   // 0010101
      54,  18,  23,  30,   // 0010110
      21,  55,  22,  31,   // 0010111
       8,  16,  25,   9,   // 0011000
       9,  27,  17,  24,   // 0011001
      10,  30,  27,  18,   // 0011010
      11,  25,  26,  19,   // 0011011
      12,  20,  29,  13,   // 0011100
      13,  31,  21,  28,   // 0011101
      14,  26,  31,  22,   // 0011110
      15,  29,  30,  23,   // 0011111
      96,  40,  33,  41,   // 0100000
      35,  97,  41,  32,   // 0100001
      38,  98,  35,  42,   // 0100010
      33,  99,  34,  43,   // 0100011
     100,  44,  37,  45,   // 0100100
      39, 101,  45,  36,   // 0100101
      34, 102,  39,  46,   // 0100110
      37, 103,  38,  47,   // 0100111
      56,  32,  41,  57,   // 0101000
      43,  57,  33,  40,   // 0101001
      58,  46,  34,  43,   // 0101010
      41,  59,  35,  42,   // 0101011
      60,  36,  45,  61,   // 0101100
      47,  61,  37,  44,   // 0101101
      62,  42,  38,  47,   // 0101110
      45,  63,  39,  46,   // 0101111
      16,  49,  56,  17,   // 0110000
      17,  51,  48,  57,   // 0110001
      18,  54,  51,  58,   // 0110010
      19,  49,  50,  59,   // 0110011
      20,  53,  60,  21,   // 0110100
      21,  55,  52,  61,   // 0110101
      22,  50,  55,  62,   // 0110110
      23,  53,  54,  63,   // 0110111
      40,  48,  57,  41,   // 0111000
      41,  59,  49,  56,   // 0111001
      42,  62,  59,  50,   // 0111010
      43,  57,  58,  51,   // 0111011
      44,  52,  61,  45,   // 0111100
      45,  63,  53,  60,   // 0111101
      46,  58,  63,  54,   // 0111110
      47,  61,  62,  55,   // 0111111
      72,  65,  73,  88,   // 1000000
      67,  73,  64,  75,   // 1000001
      74,  70,  67,  75,   // 1000010
      65,  75,  66,  73,   // 1000011
      76,  69,  77,  92,   // 1000100
      71,  77,  68,  79,   // 1000101
      78,  66,  71,  74,   // 1000110
      69,  79,  70,  77,   // 1000111
      64,  88,  73,  65,   // 1001000
      65,  75,  89,  72,   // 1001001
      66,  90,  78,  75,   // 1001010
      67,  73,  91,  74,   // 1001011
      68,  92,  77,  69,   // 1001100
      69,  79,  93,  76,   // 1001101
      70,  94,  74,  79,   // 1001110
      71,  77,  95,  78,   // 1001111
     112,  81,  88, 113,   // 1010000
      83, 113,  80,  89,   // 1010001
     114,  86,  83,  90,   // 1010010
      81, 115,  82,  91,   // 1010011
     116,  85,  92, 117,   // 1010100
      87, 117,  84,  93,   // 1010101
     118,  82,  87,  94,   // 1010110
      85, 119,  86,  95,   // 1010111
      72,  80,  89,  73,   // 1011000
      73,  91,  81,  88,   // 1011001
      74,  94,  91,  82,   // 1011010
      75,  89,  90,  83,   // 1011011
      76,  84,  93,  77,   // 1011100
      77,  95,  85,  92,   // 1011101
      78,  90,  95,  86,   // 1011110
      79,  93,  94,  87,   // 1011111
      32, 104,  97,  33,   // 1100000
      99,  33, 105,  96,   // 1100001
      34, 102,  99, 106,   // 1100010
      97,  35,  98, 107,   // 1100011
      36, 108, 101,  37,   // 1100100
     103,  37, 109, 100,   // 1100101
      38,  98, 103, 110,   // 1100110
     101,  39, 102, 111,   // 1100111
     120,  96, 105, 121,   // 1101000
     107, 121,  97, 104,   // 1101001
     122, 110,  98, 107,   // 1101010
     105, 123,  99, 106,   // 1101011
     124, 100, 109, 125,   // 1101100
     111, 125, 101, 108,   // 1101101
     126, 106, 102, 111,   // 1101110
     109, 127, 103, 110,   // 1101111
      80, 113, 120,  81,   // 1110000
      81, 115, 112, 121,   // 1110001
      82, 118, 115, 122,   // 1110010
      83, 113, 114, 123,   // 1110011
      84, 117, 124,  85,   // 1110100
      85, 119, 116, 125,   // 1110101
      86, 114, 119, 126,   // 1110110
      87, 117, 118, 127,   // 1110111
     104, 112, 121, 105,   // 1111000
     105, 123, 113, 120,   // 1111001
     106, 126, 123, 114,   // 1111010
     107, 121, 122, 115,   // 1111011
     108, 116, 125, 109,   // 1111100
     109, 127, 117, 124,   // 1111101
     110, 122, 127, 118,   // 1111110
     111, 125, 126, 119};  // 1111111

// qam256 soft demodulation nearest neighbors (p=4)
const unsigned char qam256_demod_soft_neighbors[1024] = {
       1,  16,  17,   3,   // 00000000
       0,  17,   3,  16,   // 00000001
       3,  18,   6,  19,   // 00000010
       2,  19,   1,  18,   // 00000011
      20,   5,  12,  21,   // 00000100
      21,   4,   7,  20,   // 00000101
       7,  22,   2,  23,   // 00000110
       6,  23,   5,  22,   // 00000111
       9,  24,  25,  11,   // 00001000
       8,  25,  11,  24,   // 00001001
      11,  26,  14,  27,   // 00001010
      10,  27,   9,  26,   // 00001011
      28,   4,  13,  20,   // 00001100
      29,  12,  15,  28,   // 00001101
      15,  30,  10,  31,   // 00001110
      14,  31,  13,  30,   // 00001111
       0,  17,  48,   1,   // 00010000
       1,  16,  19,  49,   // 00010001
       2,  19,  22,  50,   // 00010010
       3,  18,  17,  51,   // 00010011
       4,  21,  28,  52,   // 00010100
       5,  20,  23,  53,   // 00010101
       6,  23,  18,  54,   // 00010110
       7,  22,  21,  55,   // 00010111
       8,  25,  56,   9,   // 00011000
       9,  24,  27,  57,   // 00011001
      10,  27,  30,  58,   // 00011010
      11,  26,  25,  59,   // 00011011
      12,  20,  29,  60,   // 00011100
      13,  28,  31,  61,   // 00011101
      14,  31,  26,  62,   // 00011110
      15,  30,  29,  63,   // 00011111
      33,  48,  96,  49,   // 00100000
      32,  49,  35,  97,   // 00100001
      35,  50,  38,  98,   // 00100010
      34,  51,  33,  99,   // 00100011
      52,  37,  44, 100,   // 00100100
      53,  36,  39, 101,   // 00100101
      39,  54,  34, 102,   // 00100110
      38,  55,  37, 103,   // 00100111
      41,  56, 104,  57,   // 00101000
      40,  57,  43, 105,   // 00101001
      43,  58,  46, 106,   // 00101010
      42,  59,  41, 107,   // 00101011
      60,  36,  45, 108,   // 00101100
      61,  44,  47, 109,   // 00101101
      47,  62,  42, 110,   // 00101110
      46,  63,  45, 111,   // 00101111
      32,  49,  16,  33,   // 00110000
      33,  48,  17,  51,   // 00110001
      34,  51,  18,  54,   // 00110010
      35,  50,  19,  49,   // 00110011
      36,  53,  60,  20,   // 00110100
      37,  52,  55,  21,   // 00110101
      38,  55,  22,  50,   // 00110110
      39,  54,  53,  23,   // 00110111
      40,  57,  24,  41,   // 00111000
      41,  56,  25,  59,   // 00111001
      42,  59,  26,  62,   // 00111010
      43,  58,  27,  57,   // 00111011
      44,  52,  61,  28,   // 00111100
      45,  60,  63,  29,   // 00111101
      46,  63,  30,  58,   // 00111110
      47,  62,  61,  31,   // 00111111
      65,  80, 192,  81,   // 01000000
      64,  81, 193,  67,   // 01000001
      67,  82, 194,  70,   // 01000010
      66,  83, 195,  65,   // 01000011
      69,  76,  84, 196,   // 01000100
      68,  85, 197,  71,   // 01000101
      71,  86, 198,  66,   // 01000110
      70,  87, 199,  69,   // 01000111
      73,  88, 200,  89,   // 01001000
      72,  89, 201,  75,   // 01001001
      75,  90, 202,  78,   // 01001010
      74,  91, 203,  73,   // 01001011
      68,  77,  92, 204,   // 01001100
      76,  93, 205,  79,   // 01001101
      79,  94, 206,  74,   // 01001110
      78,  95, 207,  77,   // 01001111
      81,  64, 112,  65,   // 01010000
      80,  65, 113,  83,   // 01010001
      83,  66, 114,  86,   // 01010010
      82,  67, 115,  81,   // 01010011
      68,  85,  92, 116,   // 01010100
      69,  84,  87, 117,   // 01010101
      87,  70, 118,  82,   // 01010110
      86,  71,  85, 119,   // 01010111
      89,  72, 120,  73,   // 01011000
      88,  73, 121,  91,   // 01011001
      91,  74, 122,  94,   // 01011010
      90,  75, 123,  89,   // 01011011
      76,  84,  93, 124,   // 01011100
      77,  92,  95, 125,   // 01011101
      95,  78, 126,  90,   // 01011110
      94,  79,  93, 127,   // 01011111
      97, 112,  32, 113,   // 01100000
      96, 113,  33,  99,   // 01100001
      99, 114,  34, 102,   // 01100010
      98, 115,  35,  97,   // 01100011
     116, 101, 108,  36,   // 01100100
     117, 100, 103,  37,   // 01100101
     103, 118,  38,  98,   // 01100110
     102, 119, 101,  39,   // 01100111
     105, 120,  40, 121,   // 01101000
     104, 121,  41, 107,   // 01101001
     107, 122,  42, 110,   // 01101010
     106, 123,  43, 105,   // 01101011
     124, 100, 109,  44,   // 01101100
     125, 108, 111,  45,   // 01101101
     111, 126,  46, 106,   // 01101110
     110, 127, 109,  47,   // 01101111
      96, 113,  80,  97,   // 01110000
      97, 112,  81, 115,   // 01110001
      98, 115,  82, 118,   // 01110010
      99, 114,  83, 113,   // 01110011
     100, 117, 124,  84,   // 01110100
     101, 116,  85, 119,   // 01110101
     102, 119,  86, 114,   // 01110110
     103, 118,  87, 117,   // 01110111
     104, 121,  88, 105,   // 01111000
     105, 120,  89, 123,   // 01111001
     106, 123,  90, 126,   // 01111010
     107, 122,  91, 121,   // 01111011
     108, 116, 125,  92,   // 01111100
     109, 124,  93, 127,   // 01111101
     110, 127,  94, 122,   // 01111110
     111, 126,  95, 125,   // 01111111
     129, 144, 145, 131,   // 10000000
     128, 145, 131, 144,   // 10000001
     131, 146, 134, 147,   // 10000010
     130, 147, 129, 146,   // 10000011
     148, 133, 140, 149,   // 10000100
     149, 132, 135, 148,   // 10000101
     135, 150, 130, 151,   // 10000110
     134, 151, 133, 150,   // 10000111
     137, 152, 153, 139,   // 10001000
     136, 153, 139, 152,   // 10001001
     139, 154, 142, 155,   // 10001010
     138, 155, 137, 154,   // 10001011
     156, 132, 141, 148,   // 10001100
     157, 140, 143, 156,   // 10001101
     143, 158, 138, 159,   // 10001110
     142, 159, 141, 158,   // 10001111
     128, 145, 176, 129,   // 10010000
     129, 144, 147, 177,   // 10010001
     130, 147, 150, 178,   // 10010010
     131, 146, 145, 179,   // 10010011
     132, 149, 156, 180,   // 10010100
     133, 148, 151, 181,   // 10010101
     134, 151, 146, 182,   // 10010110
     135, 150, 149, 183,   // 10010111
     136, 153, 184, 137,   // 10011000
     137, 152, 155, 185,   // 10011001
     138, 155, 158, 186,   // 10011010
     139, 154, 153, 187,   // 10011011
     140, 148, 157, 188,   // 10011100
     141, 156, 159, 189,   // 10011101
     142, 159, 154, 190,   // 10011110
     143, 158, 157, 191,   // 10011111
     161, 176, 224, 177,   // 10100000
     160, 177, 163, 225,   // 10100001
     163, 178, 166, 226,   // 10100010
     162, 179, 161, 227,   // 10100011
     180, 165, 172, 228,   // 10100100
     181, 164, 167, 229,   // 10100101
     167, 182, 162, 230,   // 10100110
     166, 183, 165, 231,   // 10100111
     169, 184, 232, 185,   // 10101000
     168, 185, 171, 233,   // 10101001
     171, 186, 174, 234,   // 10101010
     170, 187, 169, 235,   // 10101011
     188, 164, 173, 236,   // 10101100
     189, 172, 175, 237,   // 10101101
     175, 190, 170, 238,   // 10101110
     174, 191, 173, 239,   // 10101111
     160, 177, 144, 161,   // 10110000
     161, 176, 145, 179,   // 10110001
     162, 179, 146, 182,   // 10110010
     163, 178, 147, 177,   // 10110011
     164, 181, 188, 148,   // 10110100
     165, 180, 183, 149,   // 10110101
     166, 183, 150, 178,   // 10110110
     167, 182, 181, 151,   // 10110111
     168, 185, 152, 169,   // 10111000
     169, 184, 153, 187,   // 10111001
     170, 187, 154, 190,   // 10111010
     171, 186, 155, 185,   // 10111011
     172, 180, 189, 156,   // 10111100
     173, 188, 191, 157,   // 10111101
     174, 191, 158, 186,   // 10111110
     175, 190, 189, 159,   // 10111111
     193,  64, 208,  65,   // 11000000
     192,  65, 209, 195,   // 11000001
     195,  66, 210, 198,   // 11000010
     194,  67, 211, 193,   // 11000011
      68, 197, 204, 212,   // 11000100
      69, 196, 213, 199,   // 11000101
     199,  70, 214, 194,   // 11000110
     198,  71, 215, 197,   // 11000111
     201,  72, 216,  73,   // 11001000
     200,  73, 217, 203,   // 11001001
     203,  74, 218, 206,   // 11001010
     202,  75, 219, 201,   // 11001011
      76, 196, 205, 220,   // 11001100
      77, 204, 221, 207,   // 11001101
     207,  78, 222, 202,   // 11001110
     206,  79, 223, 205,   // 11001111
     209, 192, 240, 193,   // 11010000
     208, 193, 241, 211,   // 11010001
     211, 194, 242, 214,   // 11010010
     210, 195, 243, 209,   // 11010011
     196, 213, 220, 244,   // 11010100
     197, 212, 215, 245,   // 11010101
     215, 198, 246, 210,   // 11010110
     214, 199, 213, 247,   // 11010111
     217, 200, 248, 201,   // 11011000
     216, 201, 249, 219,   // 11011001
     219, 202, 250, 222,   // 11011010
     218, 203, 251, 217,   // 11011011
     204, 212, 221, 252,   // 11011100
     205, 220, 223, 253,   // 11011101
     223, 206, 254, 218,   // 11011110
     222, 207, 221, 255,   // 11011111
     225, 240, 160, 241,   // 11100000
     224, 241, 161, 227,   // 11100001
     227, 242, 162, 230,   // 11100010
     226, 243, 163, 225,   // 11100011
     244, 229, 236, 164,   // 11100100
     245, 228, 231, 165,   // 11100101
     231, 246, 166, 226,   // 11100110
     230, 247, 229, 167,   // 11100111
     233, 248, 168, 249,   // 11101000
     232, 249, 169, 235,   // 11101001
     235, 250, 170, 238,   // 11101010
     234, 251, 171, 233,   // 11101011
     252, 228, 237, 172,   // 11101100
     253, 236, 239, 173,   // 11101101
     239, 254, 174, 234,   // 11101110
     238, 255, 237, 175,   // 11101111
     224, 241, 208, 225,   // 11110000
     225, 240, 209, 243,   // 11110001
     226, 243, 210, 246,   // 11110010
     227, 242, 211, 241,   // 11110011
     228, 245, 252, 212,   // 11110100
     229, 244, 213, 247,   // 11110101
     230, 247, 214, 242,   // 11110110
     231, 246, 215, 245,   // 11110111
     232, 249, 216, 233,   // 11111000
     233, 248, 217, 251,   // 11111001
     234, 251, 218, 254,   // 11111010
     235, 250, 219, 249,   // 11111011
     236, 244, 253, 220,   // 11111100
     237, 252, 221, 255,   // 11111101
     238, 255, 222, 250,   // 11111110
     239, 254, 223, 253};  // 11111111

// 
// amplitude/phase-shift keying (APSK)
//

// apsk4 soft demodulation nearest neighbors (p=3)
const unsigned char apsk4_demod_soft_neighbors[12] = {
       3,   1,   2,   // 00
       3,   0,   2,   // 01
       3,   0,   1,   // 10
       0,   1,   2};  // 11

// apsk8 soft demodulation nearest neighbors (p=3)
const unsigned char apsk8_demod_soft_neighbors[24] = {
       2,   3,   7,   // 000
       3,   4,   0,   // 001
       3,   6,   0,   // 010
       2,   1,   0,   // 011
       1,   5,   0,   // 100
       7,   4,   0,   // 101
       7,   2,   0,   // 110
       5,   6,   0};  // 111

// apsk16 soft demodulation nearest neighbors (p=4)
const unsigned char apsk16_demod_soft_neighbors[64] = {
       1,   4,   5,  13,   // 0000
       0,   3,   5,   2,   // 0001
       3,   6,   7,   5,   // 0010
       2,   1,   5,   7,   // 0011
      12,   0,  13,   5,   // 0100
       7,  13,   1,   0,   // 0101
       2,  14,   7,   3,   // 0110
       5,  15,   6,   2,   // 0111
      12,   9,  13,  15,   // 1000
       8,  11,  15,  13,   // 1001
      11,  14,  15,   7,   // 1010
      10,   9,  15,   8,   // 1011
       4,   8,  13,   9,   // 1100
       5,  15,  12,   4,   // 1101
       6,  10,   7,  15,   // 1110
       7,  13,  11,  10};  // 1111

// apsk32 soft demodulation nearest neighbors (p=4)
const unsigned char apsk32_demod_soft_neighbors[128] = {
       4,   1,   5,  12,   // 00000
       0,  15,   5,  13,   // 00001
       3,   6,   7,  14,   // 00010
       2,  15,   7,  13,   // 00011
       0,  24,  12,   8,   // 00100
      13,  12,  27,   1,   // 00101
      26,   2,  14,  10,   // 00110
      14,  13,  27,   3,   // 00111
      12,  28,   9,  24,   // 01000
      27,  25,   8,  12,   // 01001
      14,  30,  11,  26,   // 01010
      25,  27,  10,  14,   // 01011
       8,   5,   9,   4,   // 01100
       5,   7,  27,  15,   // 01101
       7,  10,  11,   6,   // 01110
       1,   3,  13,   5,   // 01111
      20,  17,  21,  28,   // 10000
      29,  16,  21,  31,   // 10001
      22,  19,  23,  30,   // 10010
      29,  18,  23,  31,   // 10011
      16,  24,  28,   8,   // 10100
      28,  31,  25,  17,   // 10101
      18,  26,  30,  10,   // 10110
      31,  30,  25,  19,   // 10111
      20,   4,   8,  12,   // 11000
       9,  11,  31,  23,   // 11001
       6,  22,  10,  14,   // 11010
       9,  11,  13,   5,   // 11011
       8,  21,   9,  20,   // 11100
      17,  19,  31,  23,   // 11101
      10,  23,  11,  22,   // 11110
      23,  21,  25,  29};  // 11111

// apsk64 soft demodulation nearest neighbors (p=4)
const unsigned char apsk64_demod_soft_neighbors[256] = {
       1,   8,   9,  25,   // 000000
       0,   3,   9,  11,   // 000001
       3,   7,   6,  11,   // 000010
       2,   1,  11,   6,   // 000011
       5,  12,  15,  13,   // 000100
       4,   7,  15,  14,   // 000101
      11,  14,  10,   2,   // 000110
       2,   5,  14,   6,   // 000111
       0,  24,  25,  17,   // 001000
      25,  11,  26,  27,   // 001001
      26,  30,  18,   6,   // 001010
       6,   9,  26,  10,   // 001011
       4,  28,  13,  29,   // 001100
      29,  15,  31,  12,   // 001101
      15,   6,  30,  10,   // 001110
      14,  13,  30,  31,   // 001111
      24,  48,  49,  17,   // 010000
      25,  49,  19,  24,   // 010001
      50,  22,  10,  26,   // 010010
      27,  51,  50,  17,   // 010011
      28,  52,  21,  29,   // 010100
      29,  53,  23,  55,   // 010101
      18,  54,  23,  55,   // 010110
      31,  55,  22,  29,   // 010111
      16,   8,  17,  25,   // 011000
      17,   9,  27,   8,   // 011001
      10,  27,  18,  11,   // 011010
      19,  26,  25,  50,   // 011011
      20,  12,  29,  13,   // 011100
      13,  21,  23,  28,   // 011101
      31,  10,  18,  14,   // 011110
      30,  23,  13,  22,   // 011111
      40,  33,  43,  41,   // 100000
      35,  32,  43,  42,   // 100001
      35,  39,  38,  42,   // 100010
      34,  33,  42,  38,   // 100011
      44,  37,  45,  61,   // 100100
      36,  39,  45,  47,   // 100101
      42,  47,  46,  34,   // 100110
      34,  37,  47,  38,   // 100111
      56,  32,  41,  57,   // 101000
      57,  43,  59,  40,   // 101001
      38,  43,  58,  46,   // 101010
      42,  41,  58,  59,   // 101011
      36,  60,  61,  53,   // 101100
      47,  61,  62,  63,   // 101101
      58,  62,  54,  38,   // 101110
      45,  38,  62,  46,   // 101111
      16,  56,  49,  57,   // 110000
      17,  57,  19,  51,   // 110001
      18,  54,  19,  51,   // 110010
      59,  19,  50,  57,   // 110011
      20,  60,  21,  53,   // 110100
      21,  61,  55,  60,   // 110101
      22,  50,  46,  58,   // 110110
      63,  23,  22,  53,   // 110111
      40,  48,  57,  41,   // 111000
      41,  49,  51,  56,   // 111001
      46,  59,  54,  42,   // 111010
      51,  58,  41,  50,   // 111011
      52,  44,  53,  61,   // 111100
      53,  45,  63,  44,   // 111101
      46,  63,  54,  47,   // 111110
      55,  62,  61,  22};  // 111111


// apsk128 soft demodulation nearest neighbors (p=5)
const unsigned char apsk128_demod_soft_neighbors[640] = {
       8,   1,  25,  17,   9,   // 0000000
       9,   0,  11,  25,   3,   // 0000001
      10,   3,  26,  30,   6,   // 0000010
       2,   9,  27,  26,  10,   // 0000011
       5,  12,  29,  21,  13,   // 0000100
      13,   4,  31,  29,   7,   // 0000101
       7,  10,  14,  30,   2,   // 0000110
       6,  13,  15,  14,   5,   // 0000111
      24,   0,  17,  49,  16,   // 0001000
       1,   3,  11,  27,   0,   // 0001001
       2,   6,  30,   3,   7,   // 0001010
      25,  27,  51,  26,  17,   // 0001011
      28,   4,  21,  53,  20,   // 0001100
       5,   7,  31,  15,   4,   // 0001101
      15,  30,  54,  22,  31,   // 0001110
      31,  14,  54,  23,  29,   // 0001111
      48,  24,  56,  49,   8,   // 0010000
      49,  25,  19,  57,  56,   // 0010001
      22,  51,  50,  58,  26,   // 0010010
      51,  57,  59,  43,  25,   // 0010011
      52,  28,  60,  53,  12,   // 0010100
      53,  29,  55,  61,  31,   // 0010101
      18,  54,  58,  30,  50,   // 0010110
      55,  54,  62,  63,  31,   // 0010111
       8,  16,  49,  48,   0,   // 0011000
      11,  17,  19,  51,  27,   // 0011001
      30,  27,  18,  22,  11,   // 0011010
      11,  26,  18,  51,  25,   // 0011011
      12,  20,  53,   4,  52,   // 0011100
      31,  21,  55,  23,  15,   // 0011101
      26,  14,  22,  27,  15,   // 0011110
      29,  15,  23,  14,  21,   // 0011111
      48,  96,  33,  40,  16,   // 0100000
     104,  40,  97,  56, 113,   // 0100001
      38, 102, 103, 127,  46,   // 0100010
      43, 107,  41, 106,  97,   // 0100011
      52, 100,  37,  44,  20,   // 0100100
      44, 108, 101,  60, 116,   // 0100101
      46,  34,  39, 111,  42,   // 0100110
     111,  47,  45,  38, 101,   // 0100111
      33,  56,  41,  97, 104,   // 0101000
      57,  97,  35,  43,  56,   // 0101001
      99,  46,  58,  50,  62,   // 0101010
      35,  59,  57,  99,  41,   // 0101011
      37,  60,  45, 101, 108,   // 0101100
     101,  61,  39,  47,  44,   // 0101101
      38,  42,  63,  47,  34,   // 0101110
      63,  39,  61,  46,  55,   // 0101111
      32,  16,  40,  56,  24,   // 0110000
      17,  56,  57,  40,  25,   // 0110001
      58,  59,  18,  51,  42,   // 0110010
      18,  19,  50,  59,  11,   // 0110011
      36,  20,  44,  60, 100,   // 0110100
      21,  60,  61,  29,  44,   // 0110101
      23,  22,  62,  58,  15,   // 0110110
      61,  23,  63,  47,  21,   // 0110111
      40,  49,  41,  57,  17,   // 0111000
      41,  19,  43,  49,  35,   // 0111001
      50,  62,  22,  42,  18,   // 0111010
      50,  43,  19,  99,  51,   // 0111011
      44,  53,  45,  61,  37,   // 0111100
      55,  45,  47,  53,  63,   // 0111101
      63,  58,  54,  23,  42,   // 0111110
      47,  62,  55,  46,  23,   // 0111111
      65,  72,  89,  81,  73,   // 1000000
      64,  73,  91,  89,  67,   // 1000001
      67, 110,  74,  90,  73,   // 1000010
      73,  66,  75,  74,  65,   // 1000011
      69,  76,  93,  85,  77,   // 1000100
      68,  77,  79,  93,  71,   // 1000101
     110,  71,  78,  90,  77,   // 1000110
      77,  70,  95,  78,  69,   // 1000111
      88,  64,  81, 112,  80,   // 1001000
      67,  65,  91,  75,  64,   // 1001001
      90,  75,  82,  86,  78,   // 1001010
      91,  74,  82,  83,  89,   // 1001011
      92,  68,  85, 117,  69,   // 1001100
      71,  69,  79,  95,  68,   // 1001101
      90,  95,  94,  86,  74,   // 1001110
      93,  95, 119,  78,  85,   // 1001111
      88, 120, 113, 112,  96,   // 1010000
     112,  89, 115, 121, 113,   // 1010001
      86,  83, 114, 126,  74,   // 1010010
     115,  82, 114, 122,  91,   // 1010011
     124,  92, 116, 117, 100,   // 1010100
     117,  93,  87, 109, 116,   // 1010101
      82,  94, 126,  90, 118,   // 1010110
     109, 119, 103, 127,  85,   // 1010111
      80,  72, 112, 120,  64,   // 1011000
      91,  81, 115,  83,  75,   // 1011001
      74,  78,  86,  95,  75,   // 1011010
      89,  75,  83,  81,  74,   // 1011011
      84,  76, 117, 124,  68,   // 1011100
      79,  85,  87, 119,  95,   // 1011101
     119,  86, 118, 126,  78,   // 1011110
      78,  79,  94, 119,  93,   // 1011111
     120,  32,  33, 104,  80,   // 1100000
     105,  41,  35, 107,  33,   // 1100001
     106, 102, 122, 123,  99,   // 1100010
      42, 106,  59,  43,  46,   // 1100011
      36, 124,  37, 108,  84,   // 1100100
      45, 125,  39, 111,  37,   // 1100101
      34,  98, 126, 118, 114,   // 1100110
     118, 127,  87,  34, 119,   // 1100111
      33, 113, 105,  97,  40,   // 1101000
      97, 121, 107, 123, 104,   // 1101001
      98,  99, 107,  35,  42,   // 1101010
     123,  35, 105, 106,  97,   // 1101011
     116,  37, 125, 101, 117,   // 1101100
     125,  87, 127, 117, 111,   // 1101101
      70,  66,  90,  71,  67,   // 1101110
     127,  39, 125,  38, 101,   // 1101111
     113,  81, 121, 104,  89,   // 1110000
     112, 104, 105, 121,  81,   // 1110001
     122, 126,  82,  83, 102,   // 1110010
      83, 121, 122, 123,  81,   // 1110011
     117, 108, 125, 109,  85,   // 1110100
     116,  85, 109, 108,  93,   // 1110101
     103, 126,  94, 119, 102,   // 1110110
      94,  87, 103, 118,  79,   // 1110111
      96,  80, 104, 113,  88,   // 1111000
     105, 115, 123, 112, 122,   // 1111001
     123, 114, 115,  98,  83,   // 1111010
     107, 122, 121,  98, 105,   // 1111011
      84, 100, 108, 116,  92,   // 1111100
     109, 101, 111, 127, 108,   // 1111101
     118, 114,  86, 102,  82,   // 1111110
     111, 103, 109,  34, 125};  // 1111111

// apsk256 soft demodulation nearest neighbors (p=5)
const unsigned char apsk256_demod_soft_neighbors[1280] = {
      16,   1,  49,  19,  48,   // 00000000
       3,   0,  19,  18,  49,   // 00000001
      17,   3,  22,  18,  23,   // 00000010
       1,   2,  18,  19,  22,   // 00000011
       5,  13,  28,  20,  12,   // 00000100
       4,   7,  20,  28,  21,   // 00000101
       7,  17,  23,  21,   2,   // 00000110
       6,   5,  21,  20,   4,   // 00000111
       9,  24,  57,  27,  11,   // 00001000
       8,  11,  27,  26,  57,   // 00001001
      11,  25,  30,  26,  31,   // 00001010
      10,   9,  26,  27,  30,   // 00001011
      29,  28,  60,  13,  45,   // 00001100
      15,   4,  12,  28,  29,   // 00001101
      15,  25,  31,  29,  13,   // 00001110
      14,  13,  29,  12,  25,   // 00001111
      48,   0,  33,  49,   1,   // 00010000
       2,   6,  23,  22,   3,   // 00010001
      19,  22,  54,   3,  50,   // 00010010
      18,  49,  50,   1,  35,   // 00010011
      21,  28,  52,   5,  45,   // 00010100
      20,  23,  53,   7,  52,   // 00010101
      23,  18,  55,   2,  54,   // 00010110
      22,  21,  55,  53,  17,   // 00010111
       8,  56,  41,  57,   9,   // 00011000
      14,  10,  31,  30,  11,   // 00011001
      27,  30,  62,  11,  58,   // 00011010
      26,  57,  58,   9,  43,   // 00011011
      20,  12,  45,   4,   5,   // 00011100
      12,  31,  61,  15,  60,   // 00011101
      31,  26,  63,  10,  62,   // 00011110
      30,  29,  63,  61,  25,   // 00011111
      96,  48,  51,  97, 112,   // 00100000
      49,  51,  99,  16,  48,   // 00100001
     115,  99, 114,  51,  97,   // 00100010
      50,  99,  49,  98,  38,   // 00100011
     100,  37, 119,  52,  53,   // 00100100
      39,  36, 101,  55,  53,   // 00100101
      39,  98,  50, 102, 103,   // 00100110
      37,  38,  54, 103, 101,   // 00100111
     104,  56,  59, 105, 120,   // 00101000
      59,  57, 107,  24,  56,   // 00101001
      46, 106,  58, 110, 111,   // 00101010
      58, 107,  57, 106,  42,   // 00101011
      47, 100, 108,  60,  61,   // 00101100
      52,  60, 100,  28,  20,   // 00101101
      42,  47,  62, 111, 109,   // 00101110
      44,  46, 109,  63,  61,   // 00101111
      16,  32,  33,  51,   0,   // 00110000
      33,  19,  35,   0,  16,   // 00110001
      54,  35,  38,  19,  18,   // 00110010
      97,  33,  34,  32,  99,   // 00110011
      45,  53,  20,  36,  21,   // 00110100
      55,  52,  21,  37,  23,   // 00110101
      50,  55,  39,  18,  22,   // 00110110
      53,  54,  37,  22,  23,   // 00110111
      24,  40,  41,  59, 104,   // 00111000
      27,  41,  43,   8,  24,   // 00111001
      62,  43,  42,  27,  26,   // 00111010
      41, 105, 123,  40, 107,   // 00111011
      45,  61,  12,  44,  29,   // 00111100
      63,  60,  29,  47,  31,   // 00111101
      58,  63,  46,  26,  30,   // 00111110
      61,  62,  47,  30,  31,   // 00111111
     192,  80,  65,  81, 208,   // 01000000
     193,  81,  67,  64, 192,   // 01000001
      82, 194,  70,  83,  67,   // 01000010
      83, 195,  65, 194,  66,   // 01000011
     212, 197, 132,  84, 133,   // 01000100
     215, 247,  70, 198,  86,   // 01000101
      86, 198,  66,  69,  82,   // 01000110
      87, 244, 229, 247, 245,   // 01000111
     200,  88,  73,  89, 216,   // 01001000
     201,  89, 203,  72, 200,   // 01001001
      90, 202,  78,  75, 203,   // 01001010
      91, 203,  74,  89, 121,   // 01001011
     180, 188, 252, 220, 148,   // 01001100
      79, 205, 126,  94,  95,   // 01001101
      94, 206,  74, 205,  90,   // 01001110
      77, 127, 110, 111, 126,   // 01001111
     112,  64,  81, 113,  65,   // 01010000
     113,  65,  83,  80,  67,   // 01010001
      66, 114,  86, 115,  70,   // 01010010
      67, 115,  66,  81, 113,   // 01010011
     132, 140, 220, 197,  85,   // 01010100
     197, 221, 140,  84, 141,   // 01010101
      70, 118,  82, 114, 215,   // 01010110
      71, 116, 215,  69, 199,   // 01010111
     120,  72,  89, 121,  73,   // 01011000
     121,  73,  75,  88, 203,   // 01011001
      74, 122,  94,  91,  78,   // 01011010
      75, 123,  90, 121, 105,   // 01011011
     237, 204, 172, 236, 173,   // 01011100
      95, 244, 253, 223, 237,   // 01011101
      78, 126,  90, 122,  77,   // 01011110
     116,  93,  77,  79, 205,   // 01011111
      32, 112,  97, 113,  51,   // 01100000
      51, 113, 115,  34,  96,   // 01100001
     114,  38, 102,  35, 118,   // 01100010
      34,  35,  33,  98, 114,   // 01100011
      36,  44, 117,  45,  60,   // 01100100
     103, 119,  37, 124,  39,   // 01100101
     118, 103,  38,  98, 199,   // 01100110
     102, 101,  39,  38, 199,   // 01100111
     120,  40, 105, 121,  59,   // 01101000
      59, 121,  91, 123, 104,   // 01101001
     122,  42, 110,  43, 126,   // 01101010
     123,  43,  41, 106, 122,   // 01101011
     109, 117,  44,  47, 127,   // 01101100
     111, 108,  47, 127,  46,   // 01101101
     111, 126,  42, 106,  79,   // 01101110
     109, 110,  46,  42,  79,   // 01101111
      80,  96, 113,  97,  81,   // 01110000
      81,  97, 115, 112,  83,   // 01110001
      98,  82, 118,  34,  99,   // 01110010
      34,  83,  82, 113,  97,   // 01110011
      95,  87, 125, 124, 127,   // 01110100
     119, 108, 100, 125,  36,   // 01110101
     102,  86, 114,  98, 215,   // 01110110
     117, 101,  36,  37, 124,   // 01110111
     104,  88, 121, 105,  89,   // 01111000
      89, 105,  91, 120,  75,   // 01111001
     106,  90, 126, 123, 107,   // 01111010
     107,  91, 122,  59, 105,   // 01111011
     125, 199, 101, 119, 103,   // 01111100
     124, 127, 117, 119, 108,   // 01111101
      94, 110, 122, 106,  77,   // 01111110
      79, 125, 109, 108, 111,   // 01111111
     144, 129, 145, 147, 176,   // 10000000
     131, 128, 147, 146, 145,   // 10000001
     134, 131, 150, 146, 151,   // 10000010
     129, 130, 146, 147, 150,   // 10000011
      84, 133, 148,  68, 220,   // 10000100
     132, 151, 149, 212, 148,   // 10000101
     130, 135, 151, 150, 212,   // 10000110
     134, 212, 151, 133, 130,   // 10000111
     152, 137, 153, 155, 184,   // 10001000
     139, 136, 155, 154, 153,   // 10001001
     142, 139, 158, 154, 159,   // 10001010
     137, 138, 154, 155, 158,   // 10001011
      84, 141, 156,  85, 220,   // 10001100
     159, 140, 157, 221, 156,   // 10001101
     138, 143, 159, 158, 139,   // 10001110
     221, 142, 159, 141,  85,   // 10001111
     128, 176, 177, 145, 160,   // 10010000
     147, 177, 179, 128, 144,   // 10010001
     147, 150, 182, 131, 178,   // 10010010
     145, 146, 178, 129, 179,   // 10010011
     220, 149, 132, 180, 133,   // 10010100
     148, 183, 133, 181, 151,   // 10010101
     151, 146, 183, 130, 182,   // 10010110
     150, 133, 183, 149, 134,   // 10010111
     136, 184, 185, 153, 168,   // 10011000
     185, 155, 187, 136, 152,   // 10011001
     155, 158, 190, 139, 186,   // 10011010
     154, 153, 186, 137, 187,   // 10011011
     220, 157, 140, 188, 141,   // 10011100
     191, 156, 141, 189, 159,   // 10011101
     154, 159, 191, 138, 190,   // 10011110
     141, 158, 191, 157, 142,   // 10011111
     224, 176, 161, 225, 240,   // 10100000
     225, 177, 227, 160, 163,   // 10100001
     166, 226, 230, 179, 231,   // 10100010
     179, 227, 177, 162, 226,   // 10100011
     165, 228, 181, 196, 167,   // 10100100
     164, 230, 167, 166, 245,   // 10100101
     167, 162, 178, 230, 165,   // 10100110
     166, 181, 182, 165, 164,   // 10100111
     184, 232, 169, 233, 152,   // 10101000
     233, 185, 235, 168, 171,   // 10101001
     234, 174, 238, 187, 239,   // 10101010
     187, 235, 185, 170, 234,   // 10101011
     173, 236, 189,  92, 175,   // 10101100
     172, 238, 175, 174, 237,   // 10101101
     175, 170, 186, 238, 173,   // 10101110
     174, 189, 190, 173, 172,   // 10101111
     160, 144, 177, 161, 224,   // 10110000
     145, 161, 163, 144, 176,   // 10110001
     179, 182, 166, 147, 146,   // 10110010
     163, 178, 145, 162, 166,   // 10110011
      76, 181, 228, 148, 149,   // 10110100
     167, 180, 164, 183, 149,   // 10110101
     178, 183, 167, 146, 150,   // 10110110
     182, 149, 181, 150, 151,   // 10110111
     152, 168, 185, 169, 136,   // 10111000
     153, 169, 171, 152, 184,   // 10111001
     190, 187, 174, 155, 154,   // 10111010
     171, 186, 153, 170, 174,   // 10111011
     189,  76, 236, 156, 157,   // 10111100
     188, 175, 172, 191, 157,   // 10111101
     186, 191, 175, 154, 158,   // 10111110
     157, 190, 189, 158, 159,   // 10111111
      64, 208,  65, 193,  80,   // 11000000
      65, 209, 195, 208, 192,   // 11000001
     210,  66, 198,  67, 195,   // 11000010
      67, 211, 193, 194, 210,   // 11000011
     245, 204, 164, 228, 165,   // 11000100
      85,  68,  84, 132, 140,   // 11000101
      70, 214, 194, 210,  66,   // 11000110
     124, 215, 102, 103, 118,   // 11000111
      72, 216,  73, 201,  88,   // 11001000
      73, 217, 255, 216, 200,   // 11001001
      74, 218, 206, 203, 255,   // 11001010
      75, 255,  73, 202,  74,   // 11001011
      92, 196, 252, 228, 236,   // 11001100
      77, 223,  78, 206,  94,   // 11001101
      78, 222, 202,  74, 218,   // 11001110
     248, 216, 217, 249, 201,   // 11001111
     192, 213, 193, 209,  64,   // 11010000
     241, 193, 211, 213, 195,   // 11010001
     194, 242, 214, 211, 195,   // 11010010
     195, 243, 210, 209, 241,   // 11010011
      68, 135, 133, 132, 197,   // 11010100
     240, 208, 209, 241, 193,   // 11010101
     198, 246, 210, 247, 242,   // 11010110
      69, 199, 118,  86,  87,   // 11010111
     200, 207, 201, 217,  72,   // 11011000
     249, 201, 219, 207, 255,   // 11011001
     250, 202, 222, 219, 255,   // 11011010
     251, 255, 218, 217, 249,   // 11011011
     148, 156,  76,  84, 140,   // 11011100
     143,  85, 141, 140, 142,   // 11011101
     254, 206, 218, 223, 250,   // 11011110
     253, 205, 222, 206, 254,   // 11011111
     160, 240, 225, 241, 161,   // 11100000
     241, 161, 243, 227, 224,   // 11100001
     242, 162, 231, 227, 163,   // 11100010
     243, 163, 226, 161, 225,   // 11100011
     252, 164, 180, 181, 196,   // 11100100
     245, 247, 231, 246,  71,   // 11100101
     231, 165, 162, 166, 245,   // 11100110
     246, 230, 226, 162, 229,   // 11100111
     248, 168, 233, 249, 169,   // 11101000
     249, 169, 251, 235, 232,   // 11101001
     250, 170, 239, 235, 171,   // 11101010
     251, 171, 234, 169, 233,   // 11101011
     252, 172, 188, 189,  92,   // 11101100
      92, 253, 238, 173, 239,   // 11101101
     239, 173, 170, 174, 237,   // 11101110
     238, 254, 234, 170, 253,   // 11101111
     213, 224, 241, 225, 209,   // 11110000
     225, 209, 243, 240, 211,   // 11110001
     226, 210, 246, 243, 214,   // 11110010
     211, 227, 242, 241, 225,   // 11110011
      71,  93, 204,  92, 196,   // 11110100
     229, 196, 230, 165, 231,   // 11110101
     231, 214, 242, 226, 229,   // 11110110
     229,  69, 214, 198, 246,   // 11110111
     207, 232, 249, 233, 217,   // 11111000
     233, 217, 251, 248, 219,   // 11111001
     234, 218, 254, 251, 222,   // 11111010
     235, 219, 250, 249, 233,   // 11111011
     228, 236,  76, 204, 180,   // 11111100
     223, 237, 239, 254,  93,   // 11111101
     222, 239, 250, 234, 253,   // 11111110
     203, 219, 201, 202, 218};  // 11111111


//
// specific schemes
//
